/** @mainpage Introduction

@section organisation Organisation of ApiG

In the following documentation \c APIG_ROOT will denote the root of this api (usually \c /usr/local/api_graphics).
It is organised as follows :
- \c APIG_ROOT/data : place for common data used by the api
- \c APIG_ROOT/doc : contains this documentation and the doxygen confiuration file to generate it.
- \c APIG_ROOT/examples : contains projects examples using this api.
- \c APIG_ROOT/lib : contains external libs used by classes of the api
- \c APIG_ROOT/qmake : contains \c qmake configuration files (\c .prf) for this api
- \c APIG_ROOT/src : contains the source files of the api.


@section compilation1 ApiG projects compilation
Using qmake is the simplest way to compile a project making use of ApiGraphics classes.
There is a template project file in \c APIG_ROOT.

A typical qmake project configuration file (.pro) looks like this :

@verbatim
TARGET   = Prog             # name of the produced executable
TEMPLATE = app              # make an application, not a library
CONFIG *= warn_on debug     # to get warnings and debug info
CONFIG *= apig              # necessary to use ApiG classes

HEADERS = sources//**.h       # all the C++ headers of the project
SOURCES = sources//**.cpp     # all the C++ sources of the project

OBJECTS_DIR = .obj          # the directory where will be placed object (.o) files.
@endverbatim

Once this project file is created just run the following commands in a shell, in the directory containing the \c .pro file :
@code
> qmake
> make
@endcode

The program should be compiled.


@section compilation2 How does that work?

\c qmake works with configuration files indentified by a \c .prf extension.

For each token (e.g. \c apig) put in the \c CONFIG variable,
\c qmake will search for a corresponding \c .prf file (e.g. \c apig.prf) in standard Qt locations and also in directories listed
in the \c QMAKEFEATURES environment variable (or the corresponding Qt "property variable" whose value can be queried using \c qmake \c -query or set using \c qmake \c -set \c QMAKEFEATURES \c ...).

The directory \c APIG_ROOT/qmake contains useful \c .prf files for ApiG projects configuration. So simply adding a \c CONFIG \c += \c apig line to your project file
makes automatic and transparent the necessary includes and dependencies resolution for ApiG classes.

@section code ApiG headers inclusion.

To avoid any name collisions with other's libs headers, all ApiG files are located in a \c ApiGraphics subdirectory. Only the parent of this directory is put in the \c INCLUDEPATH,
so that it makes necessary to explicitly specify the full names to include ApiG headers, e.g.:
@code
#include <ApiGraphics/ProgGLSL.h>
@endcode
Furthermore, all ApiG classe belong to the \c apig namespace.

*/
